@@ -13,10 +13,12 @@ jd_span <- function(type,d0,d1,n0,n1){
 }
 
 
-specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
+specX13_jd2r <- function(spec = NA, context_dictionary = NULL,
                          extra_info = FALSE){
 
   #Estimate
+  preliminary.check <- spec$getBasic()$isPreliminaryCheck()
+
   jestimate <-.jcall(spec,"Ljdr/spec/x13/EstimateSpec;","getEstimate")
   jest.span <-.jcall(jestimate,"Ljdr/spec/ts/SpanSelector;","getSpan")
 
@@ -42,8 +44,11 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
   jcalendar<-.jcall(jregression,"Ljdr/spec/x13/CalendarSpec;","getCalendar")
   jtd<-.jcall(jcalendar,"Ljdr/spec/x13/TradingDaysSpec;","getTradingDays")
   jeaster<-.jcall(jcalendar,"Ljdr/spec/x13/EasterSpec;","getEaster")
+  tradingdays.option <- .jcall(jtd,"S","getOption")
+  if(tradingdays.option != "UserDefined"){
+    tradingdays.option <- .jcall(jtd,"S","getTradingDays")
+  }
 
-  tradingdays.option<-.jcall(jtd,"S","getTradingDays")
   tradingdays.autoadjust<-.jcall(jtd,"Z","isAutoAdjust")
   tradingdays.leapyear <-.jcall(jtd,"S","getLengthOfPeriod")
   tradingdays.stocktd <-.jcall(jtd,"I","getW")
@@ -115,7 +120,7 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
                                                  stringsAsFactors = FALSE),
                       outliers = NA, variables = list(series = NA, description = NA))
   arima.coef.spec <- NA
-  result <- list(estimate.type = estimate.type, estimate.d0 = estimate.d0, estimate.d1 = estimate.d1,
+  result <- list(preliminary.check = preliminary.check, estimate.type = estimate.type, estimate.d0 = estimate.d0, estimate.d1 = estimate.d1,
        estimate.n0 = estimate.n0 , estimate.n1 = estimate.n1, estimate.span = estimate.span, estimate.tol = estimate.tol,
        transform.function = transform.function, transform.adjust = transform.adjust, transform.aicdiff = transform.aicdiff,
        tradingdays.option = tradingdays.option , tradingdays.autoadjust = tradingdays.autoadjust,
@@ -138,7 +143,7 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
     return(result)
 
   n_prespecified_out <- .jcall(jregression,"I","getPrespecifiedOutliersCount")
-  
+
   if(n_prespecified_out > 0 ){
     # Outlier.coef is set to TRUE: if the coefficient isn't fixed, the
     # coef value will be equal to 0 and the outlier will be estimated
@@ -162,7 +167,7 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
   }
 
   n_userdefined_var <- .jcall(jregression,"I","getUserDefinedVariablesCount")
-  
+
   if(n_userdefined_var > 0 ){
 
     # variables.coef is set to TRUE: if the coefficient isn't fixed, the
@@ -191,13 +196,13 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
                                                             row.names = var_names)
 
     # To check variables group names
-    # t <- context_dictionnary$getTsVariableManagers()
+    # t <- context_dictionary$getTsVariableManagers()
     # t$getNames()
     #
-    if(!is.null(context_dictionnary)){
+    if(!is.null(context_dictionary)){
 
       var_series <- lapply(var_names_split,function(names){
-        ts_variable <- context_dictionnary$getTsVariable(names[1],
+        ts_variable <- context_dictionary$getTsVariable(names[1],
                                                          names[2])
         ts_jd2r(ts_variable$getTsData())
       })
@@ -219,38 +224,38 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
     var_names <- sapply(var_names_split, function(x) x[2])
     var_names <- base::make.names(var_names, unique = TRUE)
     var_names <- gsub(".","_", var_names, fixed = TRUE)
-    
+
     result$userdef_spec$specification$variables <-
       TRUE
-    
+
     td_var_description <- data.frame(type = rep("Calendar",length(var_names)),
                                      coeff = 0, row.names = var_names)
-    if(is.na(result$userdef_spec$variables$description)){
+    if(identical(result$userdef_spec$variables$description, NA)){
       result$userdef_spec$variables$description <- td_var_description
     }else{
       result$userdef_spec$variables$description <- rbind(result$userdef_spec$variables$description,
                                                          td_var_description)
     }
 
-    if(!is.null(context_dictionnary)){
-      
+    if(!is.null(context_dictionary)){
+
       var_series <- lapply(var_names_split,function(names){
-        ts_variable <- context_dictionnary$getTsVariable(names[1],
+        ts_variable <- context_dictionary$getTsVariable(names[1],
                                                          names[2])
         ts_jd2r(ts_variable$getTsData())
       })
       var_series <- ts(simplify2array(var_series),
                        start = start(var_series[[1]]), frequency = frequency(var_series[[1]]))
-      if(!is.na(result$userdef_spec$variables$series)){
-        var_series <- ts.union(result$userdef_spec$variables$series)
+      if(!identical(result$userdef_spec$variables$series, NA)){
+        var_series <- ts.union(result$userdef_spec$variables$series, var_series)
       }
       if(is.mts(var_series))
         colnames(var_series) <- rownames(result$userdef_spec$variables$description)
-      
+
       result$userdef_spec$variables$series <- var_series
     }
   }
-  
+
   Phi <- jarima$getPhi()
   BPhi <- jarima$getBPhi()
   Theta <- jarima$getTheta()
@@ -270,10 +275,11 @@ specX13_jd2r <- function(spec = NA, context_dictionnary = NULL,
   result
 }
 
-specTS_jd2r<- function(spec = NA, context_dictionnary = NULL,
+specTS_jd2r<- function(spec = NA, context_dictionary = NULL,
                        extra_info = FALSE){
 
   #Estimate
+  preliminary.check <- spec$getBasic()$isPreliminaryCheck()
   jestimate <-.jcall(spec,"Ljdr/spec/tramoseats/EstimateSpec;","getEstimate")
   jest.span <-.jcall(jestimate,"Ljdr/spec/ts/SpanSelector;","getSpan")
 
@@ -303,7 +309,10 @@ specTS_jd2r<- function(spec = NA, context_dictionnary = NULL,
 
   tradingdays.mauto <-.jcall(jtd,"S","getAutomatic")
   tradingdays.pftd <-.jcall(jtd,"D","getPftd")
-  tradingdays.option <-.jcall(jtd,"S","getTradingDays")
+  tradingdays.option <- .jcall(jtd,"S","getOption")
+  if(tradingdays.option != "UserDefined"){
+    tradingdays.option <- .jcall(jtd,"S","getTradingDays")
+  }
   tradingdays.leapyear <-.jcall(jtd,"Z","getLeapYear")
   tradingdays.stocktd <-.jcall(jtd,"I","getW")
   tradingdays.test <-.jcall(jtd,"S","getRegressionTestType")
@@ -371,7 +380,8 @@ specTS_jd2r<- function(spec = NA, context_dictionnary = NULL,
                       outliers = NA, variables = list(series = NA, description = NA))
   arima.coef.spec <- NA
 
-  result <- list(estimate.type = estimate.type,estimate.d0 = estimate.d0,estimate.d1 = estimate.d1,estimate.n0 = estimate.n0,
+  result <- list(preliminary.check = preliminary.check,
+                 estimate.type = estimate.type,estimate.d0 = estimate.d0,estimate.d1 = estimate.d1,estimate.n0 = estimate.n0,
                  estimate.n1 = estimate.n1,estimate.span = estimate.span,estimate.tol = estimate.tol,estimate.eml = estimate.eml,
                  estimate.urfinal = estimate.urfinal,transform.function = transform.function,transform.fct = transform.fct,
                  tradingdays.mauto = tradingdays.mauto,tradingdays.pftd = tradingdays.pftd,tradingdays.option = tradingdays.option,
@@ -417,85 +427,85 @@ specTS_jd2r<- function(spec = NA, context_dictionnary = NULL,
 
   n_userdefined_var <- .jcall(jregression,"I","getUserDefinedVariablesCount")
   if(n_userdefined_var > 0 ){
-    
+
     # variables.coef is set to TRUE: if the coefficient isn't fixed, the
     # coef value will be equal to 0 and the variable will be estimated
     result$userdef_spec$specification$variables <-
       result$userdef_spec$specification$variables.coef <-
       TRUE
-    
+
     ud_vars <- lapply(1:n_userdefined_var, function(i){
       .jcall(jregression,
              "Ljdr/spec/ts/Utility$UserDefinedVariable;",
              "getUserDefinedVariable",
              as.integer(i-1))
     })
-    
+
     type <- sapply(ud_vars, function(x) x$getComponent())
     coeff <- sapply(ud_vars, function(x) x$getCoefficient())
     var_names <- sapply(ud_vars, function(x) x$getName())
     var_names_split <- strsplit(var_names,"[.]")
     var_names <- sapply(var_names_split, function(x) x[2])
-    
+
     result$userdef_spec$variables$description <- data.frame(type = type,
                                                             coeff = coeff,
                                                             row.names = var_names)
-    
+
     # To check variables group names
-    # t <- context_dictionnary$getTsVariableManagers()
+    # t <- context_dictionary$getTsVariableManagers()
     # t$getNames()
     #
-    if(!is.null(context_dictionnary)){
-      
+    if(!is.null(context_dictionary)){
+
       var_series <- lapply(var_names_split,function(names){
-        ts_variable <- context_dictionnary$getTsVariable(names[1],
+        ts_variable <- context_dictionary$getTsVariable(names[1],
                                                          names[2])
         ts_jd2r(ts_variable$getTsData())
       })
       var_series <- ts(simplify2array(var_series),
                        start = start(var_series[[1]]), frequency = frequency(var_series[[1]]))
-      
+
       if(is.mts(var_series))
         colnames(var_series) <- rownames(result$userdef_spec$variables$description)
-      
+
       result$userdef_spec$variables$series <- var_series
     }
-    
+
   }
-  
+
   #Calendar
   user_td <- jtd$getUserVariables()
   if(length(user_td) > 0 ){
     var_names_split <- strsplit(user_td,"[.]")
     var_names <- sapply(var_names_split, function(x) x[2])
-    
+
     result$userdef_spec$specification$variables <-
       TRUE
-    
+
     td_var_description <- data.frame(type = rep("Calendar",length(var_names)),
                                      coeff = 0, row.names = var_names)
-    if(is.na(result$userdef_spec$variables$description)){
+    if(identical(result$userdef_spec$variables$description, NA)){
       result$userdef_spec$variables$description <- td_var_description
     }else{
       result$userdef_spec$variables$description <- rbind(result$userdef_spec$variables$description,
                                                          td_var_description)
     }
 
-    if(!is.null(context_dictionnary)){
-      
+    if(!is.null(context_dictionary)){
+
       var_series <- lapply(var_names_split,function(names){
-        ts_variable <- context_dictionnary$getTsVariable(names[1],
+        ts_variable <- context_dictionary$getTsVariable(names[1],
                                                          names[2])
         ts_jd2r(ts_variable$getTsData())
       })
       var_series <- ts(simplify2array(var_series),
                        start = start(var_series[[1]]), frequency = frequency(var_series[[1]]))
-      if(!is.na(result$userdef_spec$variables$series)){
-        var_series <- ts.union(result$userdef_spec$variables$series)
+      if(!identical(result$userdef_spec$variables$series, NA)){
+        var_series <- ts.union(result$userdef_spec$variables$series, var_series)
       }
       if(is.mts(var_series))
         colnames(var_series) <- rownames(result$userdef_spec$variables$description)
-      
+
       result$userdef_spec$variables$series <- var_series
     }
   }
@@ -546,7 +556,8 @@ specX11_jd2r <- function(spec = NA, freq = NA){
   excludeFcasts <- .jcall(jx11,"Z","isExcludefcst")
 
   var <- data.frame(mode,seasonalComp,lsigma,usigma,trendAuto,trendma,seasonalma,
-                    fcasts,bcasts,excludeFcasts)
+                    fcasts,bcasts,excludeFcasts,
+                    stringsAsFactors = FALSE)
   names(var) <- sprintf("x11.%s",
                         c("mode","seasonalComp","lsigma","usigma",
                           "trendAuto","trendma","seasonalma",
@@ -620,7 +631,7 @@ preVar_r2jd <- function(jsobjct = NA, jsdict = NA, coefEna = NA,
   if (nvar == 1){
     if(n_calendar_def == 1){
       .jcall(jsdict,"V","add",varNames,ts_r2jd(series))
-      # .jcall(jtd,"V","setUserVariables", varNames)
+      .jcall(jtd,"V","setUserVariables", .jarray(paste0("r.",varNames)))
 
     }else{
       .jcall(jsdict,"V","add",varNames,ts_r2jd(series))
@@ -633,9 +644,9 @@ preVar_r2jd <- function(jsobjct = NA, jsdict = NA, coefEna = NA,
         for (i in 1:nvar){
           .jcall(jsdict,"V","add",varNames[i],ts_r2jd(series[,i]))
         }
-        # .jcall(jtd,"V","setUserVariables",
-        #        varNames[i]
-        #        )
+        .jcall(jtd,"V","setUserVariables",
+               paste0("r.",varNames)
+               )
       }else{
         for (i in 1:nvar){
           .jcall(jsdict,"V","add",varNames[i],ts_r2jd(series[,i]))
@@ -644,7 +655,7 @@ preVar_r2jd <- function(jsobjct = NA, jsdict = NA, coefEna = NA,
       }
 
     }else{
-      i_ud <- 1:nvar[-calendar_def]
+      i_ud <- (1:nvar)[-calendar_def]
       for (i in i_ud){
         .jcall(jsdict,"V","add", varNames[i],
                ts_r2jd(series[, i]))
@@ -655,10 +666,9 @@ preVar_r2jd <- function(jsobjct = NA, jsdict = NA, coefEna = NA,
       for (i in calendar_def){
         .jcall(jsdict,"V","add",varNames[i],
                ts_r2jd(series[, i]))
-        # .jcall(jtd,"V","setUserVariables",
-        #        calendar_var_names)
       }
-
+      .jcall(jtd,"V","setUserVariables",
+             .jarray(paste0("r.",varNames[calendar_def])))
     }
   }
 
@@ -730,7 +740,9 @@ specX13_r2jd <- function(rspec = NA, jdspec =NA){
   jest.span <-.jcall(jestimate,"Ljdr/spec/ts/SpanSelector;","getSpan")
   span_r2jd(jsobjct = jest.span, type = span[1,1], d0=as.character(span[1,2]), d1=as.character(span[1,3]),
             n0 = as.integer(span[1,4]), n1=as.integer(span[1,5]))
-  .jcall(jestimate ,"V","setTol", as.numeric(est[2]))
+  .jcall(jestimate ,"V","setTol", as.numeric(est["tolerance"]))
+
+  jdspec$getBasic()$setPreliminaryCheck(est[1, "preliminary.check"])
 
   #Transform
   jtransform <-.jcall(jdspec,"Ljdr/spec/x13/TransformSpec;","getTransform")
@@ -751,11 +763,19 @@ specX13_r2jd <- function(rspec = NA, jdspec =NA){
   .jcall(jregression,"V","clearUserDefinedVariables")
   jdictionary <- .jnew("jdr/spec/ts/Utility$Dictionary")
 
+
+
   #Calendar
   jcalendar<-.jcall(jregression,"Ljdr/spec/x13/CalendarSpec;","getCalendar")
   jtd<-.jcall(jcalendar,"Ljdr/spec/x13/TradingDaysSpec;","getTradingDays")
   jeaster<-.jcall(jcalendar,"Ljdr/spec/x13/EasterSpec;","getEaster")
 
+  .jcall(jeaster,"V","setJulian",as.logical(easter[2]))
+  .jcall(jeaster,"V","setDuration", as.integer(easter[3]))
+  .jcall(jeaster,"V","setTest", as.character(easter[4]))
+  .jcall(jeaster,"V","setEnabled",as.logical(easter[1]))
+
+  #Calendar options:
   if(td[1] == "UserDefined"){
     .jcall(jtd,"V","setOption","UserDefined")
   }else{
@@ -771,11 +791,6 @@ specX13_r2jd <- function(rspec = NA, jdspec =NA){
   }
   .jcall(jtd,"V","setTest",as.character(td[5]))
 
-  .jcall(jeaster,"V","setJulian",as.logical(easter[2]))
-  .jcall(jeaster,"V","setDuration", as.integer(easter[3]))
-  .jcall(jeaster,"V","setTest", as.character(easter[4]))
-  .jcall(jeaster,"V","setEnabled",as.logical(easter[1]))
-
   #user-defined specification
   if (usrspc[3]==TRUE)
     preVar_r2jd(jsobjct = jregression, jsdict = jdictionary, coefEna = usrspc[4],
@@ -846,15 +861,18 @@ specTS_r2jd <- function(rspec = NA, jdspec =NA){
   span <- s_span(rspec)
 
   #Estimate
+  jdspec$getBasic()$setPreliminaryCheck(est[1, "preliminary.check"])
+
   jestimate <-.jcall(jdspec,"Ljdr/spec/tramoseats/EstimateSpec;","getEstimate")
+
   jest.span <-.jcall(jestimate,"Ljdr/spec/ts/SpanSelector;","getSpan")
 
   span_r2jd(jsobjct = jest.span, type = span[1,1], d0=as.character(span[1,2]), d1=as.character(span[1,3]),
             n0 = as.integer(span[1,4]), n1=as.integer(span[1,5]))
 
-  .jcall(jestimate ,"V","setTol",as.numeric(est[2]))
-  .jcall(jestimate ,"V","setEml",as.logical(est[3]))
-  .jcall(jestimate ,"V","setUbp",as.numeric(est[4]))
+  .jcall(jestimate ,"V","setTol",as.numeric(est["tolerance"]))
+  .jcall(jestimate ,"V","setEml",as.logical(est["exact_ml"]))
+  .jcall(jestimate ,"V","setUbp",as.numeric(est["urfinal"]))
 
   #Transform
   jtransform <-.jcall(jdspec,"Ljdr/spec/tramoseats/TransformSpec;","getTransform")
@@ -874,6 +892,7 @@ specTS_r2jd <- function(rspec = NA, jdspec =NA){
   .jcall(jregression,"V","clearUserDefinedVariables")
   jdictionary <- .jnew("jdr/spec/ts/Utility$Dictionary")
 
+
   #Calendar
   jcalendar<-.jcall(jregression,"Ljdr/spec/tramoseats/CalendarSpec;","getCalendar")
   jtd<-.jcall(jcalendar,"Ljdr/spec/tramoseats/TradingDaysSpec;","getTradingDays")
@@ -902,10 +921,11 @@ specTS_r2jd <- function(rspec = NA, jdspec =NA){
   .jcall(jeaster,"V","setTest",as.logical(easter[4]))
   .jcall(jeaster,"V","setOption",as.character(easter[1]))
 
-
+  #user-defined specification
   if (usrspc[3]==TRUE)
     preVar_r2jd(jsobjct = jregression, jsdict = jdictionary, coefEna = usrspc[4],
                 prevar_spec = varF, jtd = jtd)
+
   #Outlier
   joutlier<-.jcall(jdspec,"Ljdr/spec/tramoseats/OutlierSpec;","getOutlier")
   joutlier.span <-.jcall(joutlier,"Ljdr/spec/ts/SpanSelector;","getSpan")
