@@ -2,15 +2,13 @@
 #'
 #' @description
 #'
-#' \code{x13_spec_def} creates (and modifies), from a predefined 'JDemetra+' model specification, a \code{c("SA_spec", "X13")} class object with the SA model specification for the X13 method.
+#' Function to create (and/or modify) a \code{c("SA_spec", "X13")} class object with the SA model specification for the X13 method. It can be done from a pre-defined 'JDemetra+' model specification (a \code{character}), a previous specification (\code{c("SA_spec", "X13")} object) or a seasonal adjustment model (\code{c("SA", "X13")} object).
 #'
-#' \code{x13_spec} creates (and/or modifies) a \code{c("SA_spec", "X13")} class object with the SA model specification for the X13 method. The object is created from a \code{c("SA", "X13")} or \code{c("SA_spec", "X13")} class object.
-#'
-#' @param spec predefined 'JDemetra+' model specification (see Details). The default is "RSA5c".
+#' @param spec model specification X13.  It can be a \code{character} of predefined X13 'JDemetra+' model specification (see \emph{Details}), an object of class \code{c("SA_spec","X13")} or an object of class \code{c("SA", "X13")}. The default is \code{"RSA5c"}.
 #'
 #' The time span of the series to be used for the estimation of the RegArima model coefficients (default from 1900-01-01 to 2020-12-31) is controlled by the following six variables: \code{estimate.from, estimate.to, estimate.first, estimate.last, estimate.exclFirst} and \code{estimate.exclLast}; where \code{estimate.from} and \code{estimate.to} have priority over remaining span control variables, \code{estimate.last} and \code{estimate.first} have priority over \code{estimate.exclFirst} and \code{estimate.exclLast}, and \code{estimate.last} has priority over \code{estimate.first}.
 #'
-#' @inheritParams regarima_spec_def_x13
+#' @inheritParams regarima_spec_x13
 #' @param x11.mode character, decomposition mode. Determines the mode of the seasonal adjustment decomposition to be performed: \code{"Undefined"} - no assumption concerning the relationship between the time series components is made; \code{"Additive"} - assumes an additive relationship; \code{"Multiplicative"} - assumes a multiplicative relationship; \code{"LogAdditive"} - performs an additive decomposition of the logarithms of the series being adjusted; \code{"PseudoAdditive"} - assumes an pseudo-additive relationship. Could be changed by the program, if needed.
 #' @param x11.seasonalComp logicals. If \code{TRUE} the program computes a seasonal component. Otherwise, the seasonal component is not estimated and its values are all set to 0 (additive decomposition) or 1 (multiplicative decomposition).
 #' @param x11.lsigma numeric, lower sigma boundary for the detection of extreme values.
@@ -24,10 +22,10 @@
 #'
 #' @details
 #'
-#' The available predefined 'JDemetra+' model specifications (for the function \code{x13_spec_def}) are described in the table below.
+#' The available predefined 'JDemetra+' model specifications are described in the table below.
 #'
 #' \tabular{rrrrrrr}{
-#' \strong{Identifier} |\tab \strong{Log/level detection} |\tab \strong{Outliers detection} |\tab \strong{Calender effects} |\tab \strong{ARIMA}\cr
+#' \strong{Identifier} |\tab \strong{Log/level detection} |\tab \strong{Outliers detection} |\tab \strong{Calendar effects} |\tab \strong{ARIMA}\cr
 #' RSA0 |\tab \emph{NA} |\tab \emph{NA} |\tab \emph{NA} |\tab Airline(+mean)\cr
 #' RSA1 |\tab automatic |\tab AO/LS/TC  |\tab \emph{NA} |\tab Airline(+mean)\cr
 #' RSA2c |\tab automatic |\tab AO/LS/TC |\tab 2 td vars + Easter |\tab Airline(+mean)\cr
@@ -38,7 +36,7 @@
 #' @return
 #' A two-elements list of class \code{c("SA_spec", "X13")}: (1) object of class \code{c("regarima_spec", "X13")} with the RegARIMA model specification, (2) object of class \code{c("X11_spec", "data.frame")} with the X11 algorithm specification.
 #' Each component refers to different part of the SA model specification, mirroring the arguments of the function (for details see arguments description).
-#' Each of the lowest-level component (except span, pre-specified outliers, user-defined variables and pre-specified ARMA coefficients) is structured within a data frame with columns denoting different variables of the model specification and rows referring to: first row - base specification, as provided within the argument \code{spec} or \code{object}; second row - user modifications as specified by the remaining arguments of the function (e.g.: \code{arima.d}); and third row - final model specification.
+#' Each of the lowest-level component (except span, pre-specified outliers, user-defined variables and pre-specified ARMA coefficients) is structured within a data frame with columns denoting different variables of the model specification and rows referring to: first row - base specification, as provided within the argument \code{spec}; second row - user modifications as specified by the remaining arguments of the function (e.g.: \code{arima.d}); and third row - final model specification.
 #' The final specification (third row) shall include user modifications (row two) unless they were wrongly specified. The pre-specified outliers, user-defined variables and pre-specified ARMA coefficients consist of a list with the \code{Predefined} (base model specification) and \code{Final} values.
 #'
 #' \item{regarima}{object of class \code{c("regarima_spec", "x13")}. See \emph{Value} of the function \code{\link{regarima_spec_x13}}}
@@ -51,16 +49,16 @@
 #' BOX G.E.P. and JENKINS G.M. (1970), "Time Series Analysis: Forecasting and Control", Holden-Day, San Francisco.
 #'
 #' BOX G.E.P., JENKINS G.M., REINSEL G.C. and LJUNG G.M. (2015), "Time Series Analysis: Forecasting and Control", John Wiley & Sons, Hoboken, N. J., 5th edition.
-#' 
+#'
 #' @seealso \code{\link{x13}}
 #'
 #' @examples \donttest{
 #' myseries <- ipi_c_eu[, "FR"]
-#' myspec1 <- x13_spec_def(spec = "RSA5c")
+#' myspec1 <- x13_spec(spec = "RSA5c")
 #' myreg1 <- x13(myseries, spec = myspec1)
-#' 
+#'
 #' # Modify a pre-specified model specification
-#' myspec2 <- x13_spec_def(spec = "RSA5c", tradingdays.option = "WorkingDays")
+#' myspec2 <- x13_spec(spec = "RSA5c", tradingdays.option = "WorkingDays")
 #' myreg2 <- x13(myseries, spec = myspec2)
 #'
 #' # Modify the model specification from a "X13" object
@@ -72,7 +70,7 @@
 #'  myreg4 <- x13(myseries, myspec4)
 #'
 #' # Pre-specified outliers
-#'  myspec1 <- x13_spec_def(spec = "RSA5c", usrdef.outliersEnabled = TRUE,
+#'  myspec1 <- x13_spec(spec = "RSA5c", usrdef.outliersEnabled = TRUE,
 #'              usrdef.outliersType = c("LS", "AO"),
 #'              usrdef.outliersDate = c("2008-10-01", "2002-01-01"),
 #'              usrdef.outliersCoef = c(36, 14),
@@ -83,24 +81,26 @@
 #'  s_preOut(myreg1)
 #'
 #'
-#' # User-defined variables
+#' # User-defined calendar regressors
 #'  var1 <- ts(rnorm(length(myseries))*10, start = start(myseries), frequency = 12)
 #'  var2 <- ts(rnorm(length(myseries))*100, start = start(myseries), frequency = 12)
 #'  var <- ts.union(var1, var2)
 #
-#'  myspec1 <- x13_spec_def(spec = "RSA5c", usrdef.varEnabled = TRUE,
-#'                          usrdef.var = var)
+#'  myspec1 <- x13_spec(spec = "RSA5c", tradingdays.option = "UserDefined",
+#'                      usrdef.varEnabled = TRUE,
+#'                      usrdef.var = var,
+#'                      usrdef.varType = c("Calendar", "Calendar"))
 #'  myreg1 <- x13(myseries, myspec1)
 #'  myreg1
 #'
-#'  myspec2 <- x13_spec_def(spec = "RSA5c", usrdef.varEnabled = TRUE,
-#'              usrdef.var = var1, usrdef.varCoef = c(2),
+#'  myspec2 <- x13_spec(spec = "RSA5c", usrdef.varEnabled = TRUE,
+#'              usrdef.var = var1, usrdef.varCoef = 2,
 #'              transform.function = "None")
 #'  myreg2 <- x13(myseries, myspec2)
 #'  s_preVar(myreg2)
 #'
 #' # Pre-specified ARMA coefficients
-#'  myspec1 <- x13_spec_def(spec = "RSA5c", automdl.enabled = FALSE,
+#'  myspec1 <- x13_spec(spec = "RSA5c", automdl.enabled = FALSE,
 #'              arima.p = 1, arima.q = 1, arima.bp = 0, arima.bq = 1,
 #'              arima.coefEnabled = TRUE,
 #'              arima.coef = c(-0.8, -0.6, 0),
@@ -111,11 +111,96 @@
 #'  myreg1
 #'
 #' # Defined seasonal filters
-#'  myspec1 <- x13_spec_def("RSA5c", x11.seasonalma = rep("S3X1", 12))
+#'  myspec1 <- x13_spec("RSA5c", x11.seasonalma = rep("S3X1", 12))
 #'  mysa1 <- x13(myseries, myspec1)
 #'}
 #' @export
-x13_spec_def <- function(spec = c("RSA5c", "RSA0", "RSA1", "RSA2c", "RSA3", "RSA4c","X11"),
+x13_spec <- function(spec = c("RSA5c", "RSA0", "RSA1", "RSA2c", "RSA3", "RSA4c","X11"),
+                     preliminary.check = NA,
+                     estimate.from = NA_character_,
+                     estimate.to = NA_character_,
+                     estimate.first = NA_integer_,
+                     estimate.last = NA_integer_,
+                     estimate.exclFirst = NA_integer_,
+                     estimate.exclLast = NA_integer_,
+                     estimate.tol = NA_integer_,
+                     transform.function = c(NA, "Auto", "None", "Log"),
+                     transform.adjust = c(NA, "None", "LeapYear", "LengthOfPeriod"),
+                     transform.aicdiff = NA_integer_,
+                     usrdef.outliersEnabled = NA,
+                     usrdef.outliersType = NA,
+                     usrdef.outliersDate = NA,
+                     usrdef.outliersCoef = NA,
+                     usrdef.varEnabled = NA,
+                     usrdef.var = NA,
+                     usrdef.varType = NA,
+                     usrdef.varCoef = NA,
+                     tradingdays.option = c(NA, "TradingDays", "WorkingDays", "UserDefined", "None"),
+                     tradingdays.autoadjust = NA,
+                     tradingdays.leapyear = c(NA, "LeapYear", "LengthOfPeriod", "None"),
+                     tradingdays.stocktd = NA_integer_,
+                     tradingdays.test = c(NA, "Remove", "Add", "None"),
+                     easter.enabled = NA,
+                     easter.julian = NA,
+                     easter.duration = NA_integer_,
+                     easter.test = c(NA, "Add", "Remove", "None"),
+                     outlier.enabled = NA,
+                     outlier.from = NA_character_,
+                     outlier.to = NA_character_,
+                     outlier.first = NA_integer_,
+                     outlier.last = NA_integer_,
+                     outlier.exclFirst = NA_integer_,
+                     outlier.exclLast = NA_integer_,
+                     outlier.ao = NA,
+                     outlier.tc = NA,
+                     outlier.ls = NA,
+                     outlier.so = NA,
+                     outlier.usedefcv = NA,
+                     outlier.cv = NA_integer_,
+                     outlier.method = c(NA, "AddOne", "AddAll"),
+                     outlier.tcrate = NA_integer_,
+                     automdl.enabled = NA,
+                     automdl.acceptdefault = NA,
+                     automdl.cancel = NA_integer_,
+                     automdl.ub1 = NA_integer_,
+                     automdl.ub2 = NA_integer_,
+                     automdl.mixed = NA,
+                     automdl.balanced = NA,
+                     automdl.armalimit = NA_integer_,
+                     automdl.reducecv = NA_integer_,
+                     automdl.ljungboxlimit = NA_integer_,
+                     automdl.ubfinal = NA_integer_,
+                     arima.mu = NA,
+                     arima.p = NA_integer_,
+                     arima.d = NA_integer_,
+                     arima.q = NA_integer_,
+                     arima.bp = NA_integer_,
+                     arima.bd = NA_integer_,
+                     arima.bq = NA_integer_,
+                     arima.coefEnabled = NA,
+                     arima.coef = NA,
+                     arima.coefType = NA,
+                     fcst.horizon = NA_integer_,
+                     x11.mode = c(NA, "Undefined", "Additive", "Multiplicative", "LogAdditive", "PseudoAdditive"),
+                     x11.seasonalComp = NA,
+                     x11.lsigma = NA_integer_,
+                     x11.usigma = NA_integer_,
+                     x11.trendAuto = NA,
+                     x11.trendma = NA_integer_,
+                     x11.seasonalma = NA_character_,
+                     x11.fcasts = NA_integer_,
+                     x11.bcasts = NA_integer_,
+                     x11.excludeFcasts = NA,
+                     object){
+  if (!missing("object")){
+    warning("'object' argument deprecated: use 'spec' instead")
+    spec <- object
+  }
+  UseMethod("x13_spec", spec)
+}
+#' @export
+x13_spec.character <- function(spec = c("RSA5c", "RSA0", "RSA1", "RSA2c", "RSA3", "RSA4c","X11"),
+                         preliminary.check = NA,
                          estimate.from = NA_character_,
                          estimate.to = NA_character_,
                          estimate.first = NA_integer_,
@@ -189,104 +274,33 @@ x13_spec_def <- function(spec = c("RSA5c", "RSA0", "RSA1", "RSA2c", "RSA3", "RSA
                          x11.seasonalma = NA_character_,
                          x11.fcasts = NA_integer_,
                          x11.bcasts = NA_integer_,
-                         x11.excludeFcasts = NA)
+                         x11.excludeFcasts = NA,
+                         object)
 {
   spec <- match.arg(spec)
   reg_spec <- gsub("RSA", "RG", spec)
-  regarima <- regarima_spec_def_x13(reg_spec,estimate.from,estimate.to,estimate.first,estimate.last,estimate.exclFirst,
-                                     estimate.exclLast,estimate.tol,transform.function,transform.adjust,
-                                     transform.aicdiff,usrdef.outliersEnabled,usrdef.outliersType,
-                                     usrdef.outliersDate,usrdef.outliersCoef,usrdef.varEnabled,usrdef.var,usrdef.varType,
-                                     usrdef.varCoef,tradingdays.option,tradingdays.autoadjust,tradingdays.leapyear,
-                                     tradingdays.stocktd,tradingdays.test,easter.enabled,easter.julian,
-                                     easter.duration,easter.test,outlier.enabled,outlier.from,outlier.to,outlier.first,
-                                     outlier.last,outlier.exclFirst,outlier.exclLast,outlier.ao,outlier.tc,outlier.ls,outlier.so,
-                                     outlier.usedefcv,outlier.cv,outlier.method,outlier.tcrate,automdl.enabled,
-                                     automdl.acceptdefault,automdl.cancel,automdl.ub1,automdl.ub2,automdl.mixed,automdl.balanced,
-                                     automdl.armalimit,automdl.reducecv,automdl.ljungboxlimit,automdl.ubfinal,arima.mu,
-                                     arima.p,arima.d,arima.q,arima.bp,arima.bd,arima.bq,arima.coefEnabled,
-                                     arima.coef,arima.coefType,fcst.horizon)
+  regarima <- regarima_spec_x13(reg_spec, preliminary.check, estimate.from,estimate.to,estimate.first,estimate.last,estimate.exclFirst,
+                                estimate.exclLast,estimate.tol,transform.function,transform.adjust,
+                                transform.aicdiff,usrdef.outliersEnabled,usrdef.outliersType,
+                                usrdef.outliersDate,usrdef.outliersCoef,usrdef.varEnabled,usrdef.var,usrdef.varType,
+                                usrdef.varCoef,tradingdays.option,tradingdays.autoadjust,tradingdays.leapyear,
+                                tradingdays.stocktd,tradingdays.test,easter.enabled,easter.julian,
+                                easter.duration,easter.test,outlier.enabled,outlier.from,outlier.to,outlier.first,
+                                outlier.last,outlier.exclFirst,outlier.exclLast,outlier.ao,outlier.tc,outlier.ls,outlier.so,
+                                outlier.usedefcv,outlier.cv,outlier.method,outlier.tcrate,automdl.enabled,
+                                automdl.acceptdefault,automdl.cancel,automdl.ub1,automdl.ub2,automdl.mixed,automdl.balanced,
+                                automdl.armalimit,automdl.reducecv,automdl.ljungboxlimit,automdl.ubfinal,arima.mu,
+                                arima.p,arima.d,arima.q,arima.bp,arima.bd,arima.bq,arima.coefEnabled,
+                                arima.coef,arima.coefType,fcst.horizon)
 
-  x11 <- x11_spec_def(spec,x11.mode,x11.seasonalComp,x11.lsigma,x11.usigma,x11.trendAuto,x11.trendma,x11.seasonalma,x11.fcasts,x11.bcasts,x11.excludeFcasts)
+  x11 <- x11_spec_def(spec, x11.mode,x11.seasonalComp,x11.lsigma,x11.usigma,x11.trendAuto,x11.trendma,x11.seasonalma,x11.fcasts,x11.bcasts,x11.excludeFcasts)
   z <- list(regarima = regarima, x11 = x11)
   class(z) <- c("SA_spec", "X13")
   return(z)
 }
-
-x11_spec_def <- function(spec = c("RSA5c", "RSA0", "RSA1", "RSA2c", "RSA3", "RSA4c","X11"),
-                          x11.mode = c(NA_character_,"Undefined","Additive","Multiplicative","LogAdditive", "PseudoAdditive"),
-                          x11.seasonalComp = NA,
-                          x11.lsigma = NA_integer_,
-                          x11.usigma = NA_integer_,
-                          x11.trendAuto = NA,
-                          x11.trendma = NA_integer_,
-                          x11.seasonalma = NA_character_,
-                          x11.fcasts = NA_integer_,
-                          x11.bcasts = NA_integer_,
-                          x11.excludeFcasts = NA)
-{
-  spec <- match.arg(spec)
-  x11.mode <- match.arg(x11.mode)
-  
-  x11.seasonalma <- spec_seasma(x11.seasonalma)
-  x11.trendma <- spec_trendma(x11.trendma)
-  
-  list.logical <- list("x11.seasonalComp", "x11.trendAuto", "x11.excludeFcasts")
-  list.numeric <- list("x11.lsigma", "x11.usigma", "x11.fcasts", "x11.bcasts")
-  
-  var.list <- list()
-  for (i in 1:length(list.logical)) {
-    eval(parse(text = paste("if( !is.logical(",list.logical[i],")) {",
-                            list.logical[i],
-                            " = NA; var.list=append(var.list,'",
-                            list.logical[i],
-                            "')}",
-                            sep = "")))
-  }
-  if (length(var.list) > 0) {
-    warning(paste("Variable(s)",
-                  deparse(as.character(var.list)),
-                  " should be logical. They are ignored."),
-            call. = FALSE)
-  }
-  
-  var.list <- list()
-  for (i in 1:length(list.numeric)) {
-    eval(parse(text = paste("if( !is.numeric(",
-                            list.numeric[i],
-                            ")) {",
-                            list.numeric[i],
-                            " = NA; var.list=append(var.list,'",
-                            list.numeric[i],
-                            "')}",
-                            sep = "")))
-  }
-  if (length(var.list) > 0) {
-    warning(paste("Variable(s)",
-                  deparse(as.character(var.list)),
-                  " should be numeric. They are ignored."),
-            call. = FALSE)
-  }
-
-  # modifed values
-  x11 <- do.call(data.frame, as.list(match.call()[c(-1, -2)]))
-  # create the java object
-  jrspec <- .jcall("jdr/spec/x13/X13Spec", "Ljdr/spec/x13/X13Spec;", "of", spec)
-  x11.spec <- specX11_jd2r(spec = jrspec)
-  # x11.spec <- do.call(data.frame, rspec)
-  # names(x11.spec) <- paste0("x11.",names(x11.spec))
-  x11.mod <- rbind(x11.spec, x11, rep(NA, length(x11.spec)))
-  z <- spec_x11(x11.mod)
-
-  class(z) <- c("X11_spec", "data.frame")
-  return(z)
-}
-#' @rdname x13_spec_def
-#' @name x13_spec_def
-#'
-#' @param object model specification, object of class \code{c("SA_spec","X13")} or \code{c("SA","X13")}.
 #' @export
-x13_spec <- function(object,
+x13_spec.X13 <- function(spec,
+                     preliminary.check = NA,
                     estimate.from = NA_character_,
                     estimate.to = NA_character_,
                     estimate.first = NA_integer_,
@@ -360,43 +374,43 @@ x13_spec <- function(object,
                     x11.seasonalma = NA_character_,
                     x11.fcasts = NA_integer_,
                     x11.bcasts = NA_integer_,
-                    x11.excludeFcasts = NA)
+                    x11.excludeFcasts = NA,
+                    object)
 {
-  if (!inherits(object, "X13") & (inherits(object, c("SA","SA_spec")) == FALSE))
+  if (!inherits(spec, c("SA","SA_spec")))
     stop("use only with c(\"SA\",\"X13\") and c(\"SA_spec\",\"X13\") objects", call. = FALSE)
 
-  regarima <- regarima_spec_x13(object, estimate.from, estimate.to, estimate.first, estimate.last, estimate.exclFirst, 
-                                estimate.exclLast, estimate.tol, transform.function, transform.adjust, 
-                                transform.aicdiff, usrdef.outliersEnabled, usrdef.outliersType, 
-                                usrdef.outliersDate, usrdef.outliersCoef, usrdef.varEnabled, usrdef.var, usrdef.varType, 
-                                usrdef.varCoef, tradingdays.option, tradingdays.autoadjust, tradingdays.leapyear, 
-                                tradingdays.stocktd, tradingdays.test, easter.enabled, easter.julian, 
-                                easter.duration, easter.test, outlier.enabled, outlier.from, outlier.to, outlier.first, 
-                                outlier.last, outlier.exclFirst, outlier.exclLast, outlier.ao, outlier.tc, outlier.ls, outlier.so, 
-                                outlier.usedefcv, outlier.cv, outlier.method, outlier.tcrate, automdl.enabled, 
-                                automdl.acceptdefault, automdl.cancel, automdl.ub1, automdl.ub2, automdl.mixed, automdl.balanced, 
-                                automdl.armalimit, automdl.reducecv, automdl.ljungboxlimit, automdl.ubfinal, arima.mu, 
-                                arima.p, arima.d, arima.q, arima.bp, arima.bd, arima.bq, arima.coefEnabled, 
-                                arima.coef, arima.coefType, fcst.horizon)
-  
-  x11 <- x11_spec(object, x11.mode, x11.seasonalComp, x11.lsigma, x11.usigma, x11.trendAuto,
+  regarima <- regarima_spec_x13(spec, preliminary.check, estimate.from, estimate.to, estimate.first, estimate.last, estimate.exclFirst,
+                                estimate.exclLast, estimate.tol, transform.function, transform.adjust,
+                                transform.aicdiff, usrdef.outliersEnabled, usrdef.outliersType,
+                                usrdef.outliersDate, usrdef.outliersCoef, usrdef.varEnabled, usrdef.var, usrdef.varType,
+                                usrdef.varCoef, tradingdays.option, tradingdays.autoadjust, tradingdays.leapyear,
+                                tradingdays.stocktd, tradingdays.test, easter.enabled, easter.julian,
+                                easter.duration, easter.test, outlier.enabled, outlier.from, outlier.to, outlier.first,
+                                outlier.last, outlier.exclFirst, outlier.exclLast, outlier.ao, outlier.tc, outlier.ls, outlier.so,
+                                outlier.usedefcv, outlier.cv, outlier.method, outlier.tcrate, automdl.enabled,
+                                automdl.acceptdefault, automdl.cancel, automdl.ub1, automdl.ub2, automdl.mixed, automdl.balanced,
+                                automdl.armalimit, automdl.reducecv, automdl.ljungboxlimit, automdl.ubfinal, arima.mu,
+                                arima.p, arima.d, arima.q, arima.bp, arima.bd, arima.bq, arima.coefEnabled, arima.coef, arima.coefType, fcst.horizon)
+
+  x11 <- x11_spec(spec, x11.mode, x11.seasonalComp, x11.lsigma, x11.usigma, x11.trendAuto,
                   x11.trendma, x11.seasonalma, x11.fcasts, x11.bcasts, x11.excludeFcasts)
   z <- list(regarima = regarima, x11 = x11)
   class(z) <- c("SA_spec", "X13")
   return(z)
 }
 
-x11_spec <- function(object,
-                          x11.mode = c(NA_character_,"Undefined","Additive","Multiplicative","LogAdditive", "PseudoAdditive"),
-                          x11.seasonalComp = NA,
-                          x11.lsigma = NA_integer_,
-                          x11.usigma = NA_integer_,
-                          x11.trendAuto = NA,
-                          x11.trendma = NA_integer_,
-                          x11.seasonalma = NA_character_,
-                          x11.fcasts = NA_integer_,
-                          x11.bcasts = NA_integer_,
-                          x11.excludeFcasts = NA)
+x11_spec <- function(spec,
+                     x11.mode = c(NA_character_,"Undefined","Additive","Multiplicative","LogAdditive", "PseudoAdditive"),
+                     x11.seasonalComp = NA,
+                     x11.lsigma = NA_integer_,
+                     x11.usigma = NA_integer_,
+                     x11.trendAuto = NA,
+                     x11.trendma = NA_integer_,
+                     x11.seasonalma = NA_character_,
+                     x11.fcasts = NA_integer_,
+                     x11.bcasts = NA_integer_,
+                     x11.excludeFcasts = NA)
 {
   x11.mode <- match.arg(x11.mode)
 
@@ -443,7 +457,7 @@ x11_spec <- function(object,
 
   # modifed values
   x11 <- do.call(data.frame, as.list(match.call()[c(-1, -2)]))
-  x11.spec <- s_x11(object)
+  x11.spec <- s_x11(spec)
   x11.mod <- rbind(x11.spec, x11, rep(NA, length(x11.spec)))
   z <- spec_x11(x11.mod)
 
@@ -451,5 +465,72 @@ x11_spec <- function(object,
   return(z)
 }
 
+x11_spec_def <- function(spec = c("RSA5c", "RSA0", "RSA1", "RSA2c", "RSA3", "RSA4c","X11"),
+                         x11.mode = c(NA_character_,"Undefined","Additive","Multiplicative","LogAdditive", "PseudoAdditive"),
+                         x11.seasonalComp = NA,
+                         x11.lsigma = NA_integer_,
+                         x11.usigma = NA_integer_,
+                         x11.trendAuto = NA,
+                         x11.trendma = NA_integer_,
+                         x11.seasonalma = NA_character_,
+                         x11.fcasts = NA_integer_,
+                         x11.bcasts = NA_integer_,
+                         x11.excludeFcasts = NA)
+{
+  spec <- match.arg(spec)
+  x11.mode <- match.arg(x11.mode)
 
+  x11.seasonalma <- spec_seasma(x11.seasonalma)
+  x11.trendma <- spec_trendma(x11.trendma)
+
+  list.logical <- list("x11.seasonalComp", "x11.trendAuto", "x11.excludeFcasts")
+  list.numeric <- list("x11.lsigma", "x11.usigma", "x11.fcasts", "x11.bcasts")
+
+  var.list <- list()
+  for (i in 1:length(list.logical)) {
+    eval(parse(text = paste("if( !is.logical(",list.logical[i],")) {",
+                            list.logical[i],
+                            " = NA; var.list=append(var.list,'",
+                            list.logical[i],
+                            "')}",
+                            sep = "")))
+  }
+  if (length(var.list) > 0) {
+    warning(paste("Variable(s)",
+                  deparse(as.character(var.list)),
+                  " should be logical. They are ignored."),
+            call. = FALSE)
+  }
+
+  var.list <- list()
+  for (i in 1:length(list.numeric)) {
+    eval(parse(text = paste("if( !is.numeric(",
+                            list.numeric[i],
+                            ")) {",
+                            list.numeric[i],
+                            " = NA; var.list=append(var.list,'",
+                            list.numeric[i],
+                            "')}",
+                            sep = "")))
+  }
+  if (length(var.list) > 0) {
+    warning(paste("Variable(s)",
+                  deparse(as.character(var.list)),
+                  " should be numeric. They are ignored."),
+            call. = FALSE)
+  }
+
+  # modifed values
+  x11 <- do.call(data.frame, as.list(match.call()[c(-1, -2)]))
+  # create the java object
+  jrspec <- .jcall("jdr/spec/x13/X13Spec", "Ljdr/spec/x13/X13Spec;", "of", spec)
+  x11.spec <- specX11_jd2r(spec = jrspec)
+  # x11.spec <- do.call(data.frame, rspec)
+  # names(x11.spec) <- paste0("x11.",names(x11.spec))
+  x11.mod <- rbind(x11.spec, x11, rep(NA, length(x11.spec)))
+  z <- spec_x11(x11.mod)
+
+  class(z) <- c("X11_spec", "data.frame")
+  return(z)
+}
 
